/*!
  @file
  @author Shin'ichiro Nakaoka
*/

#ifndef CNOID_BODYPLUGIN_CONTROLLER_ITEM_H_INCLUDED
#define CNOID_BODYPLUGIN_CONTROLLER_ITEM_H_INCLUDED

#include <cnoid/Item>
#include "exportdecl.h"

namespace cnoid {

    class Body;

    class CNOID_EXPORT ControllerItem : public Item
    {
      public:
        ControllerItem();
        ControllerItem(const ControllerItem& org);
        virtual ~ControllerItem();

        /**
           Override this function to control a body
           @note This function is called from the main thread.
        */
        virtual bool initialize(Body* body, double worldTimeStep);

        /**
           Override this function to do general processing which is not related with a particular body
           @note This function is called from the main thread.
        */
        virtual bool initialize(double worldTimeStep);
        
        virtual double timeStep() = 0;

        /**
           @note This function is called from the simulation thread.
        */
        virtual void input() = 0;

        /*
          @return false to request stopping
          @note The body oject given in the initalized function() must not be accessed
          in this function. The access should be done in input() and output().
           @note This function is called from the simulation thread.
        */
        virtual bool control() = 0;
        
        /**
           @note This function is called from the simulation thread.
        */
        virtual void output() = 0;
        
        /**
           @note This function is called from the main thread.
        */
        virtual void finalize();

        SignalProxy< boost::signal<void(const std::string& message)> > sigMessage();

        std::string getMessage();

      protected:
        void putMessage(const std::string& message);

      private:
        std::string message_;
        boost::signal<void(const std::string& message)> sigMessage_;
    };
        
    typedef boost::intrusive_ptr<ControllerItem> ControllerItemPtr;
}

#endif
