/**
  @author Shin'ichiro Nakaoka
*/

#ifndef CNOID_SIMPLE_CONTROLLER_SIMPLE_CONTROLLER_H_INCLUDED
#define CNOID_SIMPLE_CONTROLLER_SIMPLE_CONTROLLER_H_INCLUDED

#include <cnoid/Body>
#include <ostream>

namespace cnoid {

    class SimpleController
    {
      public:
	typedef SimpleController* (*Factory)();
        
        virtual ~SimpleController() { }

        virtual bool initialize(Body* body, double timeStep, std::ostream& os) = 0;

        /*
          @return false to request stopping
        */
        virtual bool control(Body* body, std::ostream& os) = 0;
    };

}


#if defined(WIN32) || defined(_WIN32) || defined(__WIN32__) || defined(__NT__)

extern "C" __declspec(dllexport) cnoid::SimpleController* createSimpleController();

#define CNOID_IMPLEMENT_SIMPLE_CONTROLLER_FACTORY(ControllerClassName) \
    extern "C" __declspec(dllexport) cnoid::SimpleController* createSimpleController() \
    {                                    \
        return new ControllerClassName();      \
    }
#else

extern "C" cnoid::SimpleController* createSimpleController();

#define CNOID_IMPLEMENT_SIMPLE_CONTROLLER_FACTORY(ControllerClassName) \
    extern "C" cnoid::SimpleController* createSimpleController() \
    {                                    \
        return new ControllerClassName();      \
    }
#endif

#endif
