/**
   @file
   @author Shin'ichiro Nakaoka
*/

#include "MultiVector3Seq.h"
#include "ValueTree.h"
#include "YAMLWriter.h"
#include "gettext.h"

using namespace std;
using namespace cnoid;

MultiVector3Seq::MultiVector3Seq(int numParts, int numFrames, double frameRate)
    : BaseSeqType("MultiVector3Seq", numParts, numFrames, frameRate)
{

}
        

MultiVector3Seq::MultiVector3Seq(const MultiVector3Seq& org)
    : BaseSeqType(org)
{

}


MultiVector3Seq& MultiVector3Seq::operator=(const MultiVector3Seq& rhs)
{
    if(this != &rhs){
        BaseSeqType::operator=(rhs);
    }
    return *this;
}
    

MultiVector3Seq::~MultiVector3Seq()
{

}
        

bool MultiVector3Seq::doWriteSeq(YAMLWriter& writer)
{
    if(BaseSeqType::doWriteSeq(writer)){
        writer.putKey("frames");
        writer.startSequence();
        const int m = numParts();
        const int n = numFrames();
        for(int i=0; i < n; ++i){
            Frame f = frame(i);
            writer.startFlowStyleSequence();
            for(int j=0; j < m; ++j){
                writer.startFlowStyleSequence();
                const Vector3& p = f[j];
                writer.putScalar(p.x());
                writer.putScalar(p.y());
                writer.putScalar(p.z());
                writer.endSequence();
            }
            writer.endSequence();
        }
        writer.endSequence();
        return true;
    }
    return false;
}


bool MultiVector3Seq::doReadSeq(const Mapping& archive)
{
    if(BaseSeqType::doReadSeq(archive)){
        const string& type = archive["type"].toString();
        if(type == seqType()){
            const Sequence& values = *archive.findSequence("frames");
            if(values.isValid()){
                const int nParts = archive["numParts"].toInt();
                const int nFrames = values.size();
                setDimension(nFrames, nParts);
                
                for(int i=0; i < nFrames; ++i){
                    const Sequence& frameNode = *values[i].toSequence();
                    Frame f = frame(i);
                    const int n = std::min(frameNode.size(), nParts);
                    for(int j=0; j < n; ++j){
                        const Sequence& node = *frameNode[j].toSequence();
                        if(node.size() == 3){
                            f[j] << node[0].toDouble(), node[1].toDouble(), node[2].toDouble();
                        } else {
                            node.throwException("Element is not a three dimension vector");
                        }
                    }
                }
            }
        }
        return true;
    }
    return false;
}
